#*************************************************************************/
#**
#* \file nwcpy.py
#* \brief Python visualizer for NWC/GEO image-like products
#*
#* \par Copyright 2017, EUMETSAT, All Rights Reserved
#*
#* \par PRODUCED BY :  GMV Aerospace and Defence S.A.U.
#*                     This SW was developed by GMV within the
#*                     context of the EUMETSAT Satellite Application
#*                     Facility Co-operation Agreement for Support to
#*                     Nowcasting and Very Short Range Forecasting,
#*                     dated on 29 June 2011 between EUMETSAT
#*                     and AEMET.
#*
#* \par PROJECT     :  NWC SAF
#*
#* \par UNIT NAME   :  NWCPY
#*
#* \par FILE        :  nwcpy.py
#*
#* \par TYPE        :  source
#*
#* \par FUNCTION    :  
# Usage: python nwcpy.py <path/file.nc> <variable1> <width> <height>
#
# This script plots meteorological fields from image-like products in netCDF format: CMA, CT, CTTH, CMIC, PC, CRR,
# PPh, iSHAI, ASII-NG, CI and EXIM. The complete path to the nc file to be plotted must be introduced. The final
# image is named <file>.<variable1>.gif and stored in the working directory, its size set to <width> x <height>. If
# 0 0 is introduced, the default dimensions (1100 x 800) are used.
# The variable <variable1> is read from the nc file and plotted using the corresponding palette and attributes.
# Background image for CI product is taken from the IR120-BT file from L1SD product in the same path replacing CI
# by L1SD. iSHAI product is plotted using iSHAI IR_band variable for the background.
#
# Example:
#
# execute:      python nwcpy.py /home/NWCGEO/export/CMA/S_NWC_CMA_MSG3_Spain-VISIR_20151013T144500Z.nc cma_cloudsnow 0 0
# output file:  S_NWC_CMA_MSG3_Spain-VISIR_20151013T144500Z.cma_cloudsnow.gif
# in:           . (working directory)
#
#**************************************************************************/
#* HISTORY
#*
#*  DATE      VERSION     AUTHOR     REASONS
#* SEP 2017    1.0       AEMET/GMV   First version
#* MAR 2018    2.0       AEMET/GMV   Operational
#*
#****************************************************************************/


import matplotlib
matplotlib.use('Agg')

import netCDF4
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.pyplot import cm
from matplotlib.offsetbox import (OffsetImage, AnnotationBbox)
from mpl_toolkits.basemap import Basemap
import os
from os import path
import subprocess
import re
import sys
import datetime

# Current date and time
now = datetime.datetime.now()

# Usage
if len(sys.argv) != 5:
    print "       Usage: python nwcpy.py <path/file.nc> <variable1> <width> <height>"
    print "       Examples: python nwcpy.py /home/NWCGEO/export/CMA/S_NWC_CMA_MSG3_Spain-VISIR_20151013T144500Z.nc cma_cloudsnow 0 0"
    print "                 python nwcpy.py /home/NWCGEO/export/L1SD/S_NWC_IR120-BT_MSG3_MSG-N-VISIR_20151013T144500Z.nc data 1200 700"
    quit(2)

# Input and output directories
inDir = os.path.dirname(sys.argv[1])
print "inDir=", inDir
outDir = os.getcwd()
print "outDir=", outDir
# Input file
file = os.path.basename(sys.argv[1])

# ----------------------------------------------- #
# Renames input file and opens the data matrices  #
# ----------------------------------------------- #
# Builds complete input and output paths
filename = os.path.join(inDir, file)
print 'File', filename
fileType = file
pge = fileType.split('_', 3)[2]
if '_0' in fileType:
    fileType = "EXIM_" + pge
else:
    fileType = pge
print "fileType=", fileType

# Opens the netCDF product
prodNC4 = netCDF4.Dataset(filename, 'r')

# Reads input variables
vars = {}
vars[fileType] = [sys.argv[2]]

# --------------------------------------------- #
# Creates image names in output directory
# --------------------------------------------- #
for ivar in range(len(vars[fileType])):
    title = file
    title = title.replace('.nc', '')
    outFile = outDir + '/' + title + '.' + vars[fileType][ivar] + '.gif'
    print "Creating ", outFile

    print "Variable ", vars[fileType][ivar], "--------------------------------"

    # -------------------------------------- #
    # Reads data variable and its attributes
    # -------------------------------------- #
    prod = prodNC4.variables[vars[fileType][ivar]][:]
    print "   data type:", prod.dtype, "| max =", np.amax(prod), " min =", np.amin(prod)
    print "   prod size", prod.shape
    prodVar = prodNC4.variables[vars[fileType][ivar]]
    validRange = prodVar.getncattr("valid_range")
    print "   validRange", validRange[0], " ", validRange[1]
    prodName = prodVar.getncattr("long_name")
    if "units" in prodVar.__dict__:
        prodUnit = prodVar.getncattr("units")
        if prodUnit == "unitless":
            unitStr = ''
        elif prodUnit == '1':
            unitStr = ' (%)'
        else:
            unitStr = ' (' + prodUnit + ')'
    else:
        unitStr = ''   
    fillValue = prodVar.getncattr("_FillValue")
    attributes = prodVar.__dict__
    if "scale_factor" in attributes:
        scale = prodVar.getncattr("scale_factor")
        flagClass = False
    else:
        flagClass = True
    if "add_offset" in attributes:
        offset = prodVar.getncattr("add_offset")
    if "flag_meanings" in attributes:
        flag_meanings = prodVar.getncattr("flag_meanings").split()
        print "   flag_meanings", flag_meanings
        flagClass = True
        if flag_meanings[0] != 'undefined':
            flagUndef = False
            for istr in range(len(flag_meanings)):
                flag_meanings[istr] = flag_meanings[istr].replace('_', ' ')
            flag_values = prodVar.getncattr("flag_values")
            print "   flag_values", flag_values
            bounds = np.append(flag_values, flag_values[-1] + 1)
        else:
            flagUndef = True
            bounds = np.arange(validRange[0], validRange[1] + 2)
            if (bounds.size < 20):
                print "   no step"
                flag_values = np.arange(validRange[0], validRange[1] + 1)
            else:
                step = (validRange[1] - validRange[0]) / 20
                print "   step", step
                flag_values = np.arange(validRange[0], validRange[1] + 1, step)
    else:
        flagUndef = True
        bounds = np.arange(validRange[0], validRange[1] + 2)
        if (bounds.size < 20):
            print "   no step"
            flag_values = np.arange(validRange[0], validRange[1] + 1)
        else:
            step = (validRange[1] - validRange[0]) / 20
            print "   step", step
            flag_values = np.arange(validRange[0], validRange[1] + 1, step)
#    print "   bounds", bounds

    # -------------------------------------------------------- #
    # Reads ancillary variables (not included in L1SD product) #
    # -------------------------------------------------------- #
    if "L1SD" in filename:
        prodCmap = cm.Greys
        flagClass = False
    else:
        ancillary = prodVar.getncattr("ancillary_variables").split()

        # Reads color palette
        palVar = {}
        matchPal = [s for s in ancillary if "_pal" in s]
        palVar[fileType] = matchPal[0]
        prodPal = prodNC4.variables[palVar[fileType]][:]
        prodPal = prodPal / 255.0
        # Removes black color in crrph_iqf and pcph palettes
        if "iqf" in vars[fileType][ivar]:
            prodPal = np.vstack((prodPal[0:100,:], prodPal[255,:]))
        elif "pcph" in vars[fileType][ivar]:
            prodPal = np.vstack((prodPal[0:100,:], prodPal[255,:]))
        print "   palette", palVar

        # Creates color map from palette array
        gamma = np.ones((prodPal.shape[0], 1))
        prodPal2 = np.concatenate((prodPal, gamma), axis=1)
        cmap = plt.cm.jet
        cmaplist = [prodPal2[i, :] for i in range(prodPal2.shape[0])]
        prodCmap = cmap.from_list('prod map', cmaplist, 256)

    # ------------------------ #
    # Image frame dimensions   #
    # ------------------------ #
    if sys.argv[3] == '0':
        width = float(1100)
        height = float(800)
    else:
        width = float(sys.argv[3])
        height = float(sys.argv[4])
    print "   wxh ", width, "x", height

    # --------------------------------------------- #
    # Reads gdal projection parameters from netCDF  #
    # --------------------------------------------- #
    geodef = prodNC4.getncattr('gdal_projection')
    proj = str(geodef[6:10])
    a = float(geodef[14:23])
    b = float(geodef[32:41])
    lon_0 = float(geodef[54])
    h = float(geodef[66:74])
   
    lowleftx = prodNC4.getncattr('gdal_xgeo_up_left')
    uprighty = prodNC4.getncattr('gdal_ygeo_up_left')
    uprightx = prodNC4.getncattr('gdal_xgeo_low_right')
    lowleftty = prodNC4.getncattr('gdal_ygeo_low_right')

    # ----------------------- #
    # Plots the image frame   #
    # ----------------------- #
    myDpi = 100  # image resolution
    fig, ax = plt.subplots(figsize=(width / myDpi, height / myDpi), dpi=myDpi)

    # Includes NWCSAF logo in the image
    imNWCSAF = plt.imread(os.environ['NWCPY_PATH'] + '/logoNWCSAF.png')
    imboxNWCSAF = OffsetImage(imNWCSAF, zoom=width/1100-width/2000)
    imboxNWCSAF.image.axes = ax
    abNWCSAF = AnnotationBbox(imboxNWCSAF, xy=(0.998, 0.996),
                              xycoords='axes fraction',
                              boxcoords="offset points",
                              pad=0, box_alignment=(1, 1), frameon=False)
    ax.add_artist(abNWCSAF)

    # Map projection definition
    m = Basemap(lon_0=lon_0, llcrnrx=lowleftx, llcrnry=lowleftty, urcrnrx=uprightx, urcrnry=uprighty,
                projection=proj, resolution='l', area_thresh=250, satellite_height=h, rsphere=(a, b))
    m.drawcoastlines(linewidth=1.3, color='white')
    m.drawcountries(linewidth=0.8, color='white')

    # Max and min plotted values
    if flagClass:
        Vmin = 0
        Vmax = fillValue
    elif 'L1SD' in filename:
        Vmin = np.amin(prod)
        Vmax = np.amax(prod)
    elif 'iqf' in vars[fileType][ivar]:
        Vmin = validRange[0] * scale + offset
        Vmax = validRange[1] * scale + offset + 1.2
    else:
        Vmin = validRange[0] * scale + offset
        Vmax = validRange[1] * scale + offset

    # L1SD IR120 background for CI product
    if "CI" in file:
        l1sdFile = filename.replace('CI', 'L1SD')
        l1sdFile = l1sdFile.replace('_L1SD_', '_IR120-BT_')
        print "L1SD file", l1sdFile, "----------"
        l1sdNC4 = netCDF4.Dataset(l1sdFile, 'r')
        prodL1SD = l1sdNC4.variables['data'][:]
        print "  data type:", prodL1SD.dtype, "| max =", np.amax(prodL1SD), " min =", np.amin(prodL1SD)
        prodVarL1SD = l1sdNC4.variables['data']
        # L1SD palette (inverse grayscale)
        l1sdCmap = cm.Greys
        VminL1SD = np.amin(prodL1SD)
        VmaxL1SD = np.amax(prodL1SD)
        # Plots IR120 band from L1SD
        m.imshow(prodL1SD, origin='upper', vmin=VminL1SD, vmax=VmaxL1SD, cmap=l1sdCmap, alpha=0.8)

    # Replaces NaN (--) values with Vmax to plot them properly
    # Not applied to CI (transparency)
    if "CI" not in file:
        prod_type = str(type(prod))
        if re.search('MaskedArray', prod_type):
            prod2 = prod.filled(Vmax)
        else:
            prod2 = prod
    else:
        prod2 = prod

    # ------------------ #
    # Product plotting   #
    # ------------------ #
    im = m.imshow(prod2, origin='upper', vmin=Vmin, vmax=Vmax, cmap=prodCmap)

    # Title and text labels
    plt.title(title, size=(width/1100)*8)
    plt.xlabel(prodName + unitStr, size=(width/1100)*11)
    bbox_props = dict(boxstyle="square", fc='white', lw=0.5)
    plt.annotate(s='COPYRIGHT ' + now.strftime("%Y") + ", EUMETSAT. All rights reserved", xy=(0.003, 0.994),
                 size=(width/1100)*5, xycoords="axes fraction", family='monospace', bbox=bbox_props, horizontalalignment='left',
                 verticalalignment='top')
    plt.minorticks_on()

    # Plots colorbar and ticklabels
    if flagClass:
        cbar = plt.colorbar(im, boundaries=bounds, ticks=flag_values + 0.5, shrink=1.0, orientation='horizontal',
                            pad=0.05, aspect=30)
        if not flagUndef:
            cbar.ax.set_xticklabels(flag_meanings, size=(width/1100)*10, rotation=20, horizontalalignment='right')
        else:
            cbar.ax.set_xticklabels(flag_values, size=(width/1100)*10, rotation=20, horizontalalignment='right')
    else:
        if "iqf" in vars[fileType][ivar]:
            cbar = plt.colorbar(im, ticks=(0,20,40,60,80,100), shrink=1.0, orientation='horizontal', pad=0.05, aspect=30)
            cbar.ax.tick_params(labelsize=(width/1100)*10)
        elif "pcph" in vars[fileType][ivar]:
            cbar = plt.colorbar(im, ticks=(0,20,40,60,80,100), shrink=1.0, orientation='horizontal', pad=0.05, aspect=30)
            cbar.ax.tick_params(labelsize=(width/1100)*10)
        else:
            cbar = plt.colorbar(im, shrink=1.0, orientation='horizontal', pad=0.05, aspect=30)
            cbar.ax.tick_params(labelsize=(width/1100)*10)

    # iSHAI IR_band background for iSHAI product
    if 'iSHAI' in file:
        prodIR = prodNC4.variables['IR_band'][:]
        prodVarIR = prodNC4.variables['IR_band']
        fillValueIR = prodVarIR.getncattr("_FillValue")
        # iSHAI IR_band palette
        ishaiPal = prodNC4.variables['IR_band_pal'][:]
        ishaiPal = ishaiPal / 255.0
        gammaIR = np.ones((ishaiPal.shape[0], 1))
        ishaiPal2 = np.concatenate((ishaiPal, gammaIR), axis=1)
        # Colormap
        ishaiCmaplist = [ishaiPal2[i, :] for i in range(ishaiPal2.shape[0])]
        ishaiCmap = cmap.from_list('ishai map', ishaiCmaplist, ishaiPal2.shape[0])
        # Boundaries and plotting
        VminIR = 0
        VmaxIR = fillValueIR
        m.imshow(prodIR, origin='upper', vmin=VminIR, vmax=VmaxIR, cmap=ishaiCmap)

    # -------------------------------------- #
    # Saves the image and closes the figure  #
    # -------------------------------------- #
    print "Writing ", outFile
    plt.savefig(outFile, format='png', dpi=myDpi, bbox_inches='tight')  # Python does not support GIF format
    plt.close()
    # Conversion to GIF calling ImageMagick
    subprocess.call("convert " + outFile + " gif:" + outFile, shell=True)
    quit()
