#*************************************************************************/
#**
#* \file nwcpy_asii.py
#* \brief Python visualizer for the NWC/GEO ASII product
#*
#* \par Copyright 2017, EUMETSAT, All Rights Reserved
#*
#* \par PRODUCED BY :  GMV Aerospace and Defence S.A.U.
#*                     This SW was developed by GMV within the
#*                     context of the EUMETSAT Satellite Application
#*                     Facility Co-operation Agreement for Support to
#*                     Nowcasting and Very Short Range Forecasting,
#*                     dated on 29 June 2011 between EUMETSAT
#*                     and AEMET.
#*
#* \par PROJECT     :  NWC SAF
#*
#* \par UNIT NAME   :  NWCPY
#*
#* \par FILE        :  nwcpy_asii.py
#*
#* \par TYPE        :  source
#*
#* \par FUNCTION    :
# Usage: python nwcpy_asii.py <path/file.bufr> <width> <height>
#
# This script plots conceptual model phenomena from an ASII product in bufr format. The complete path to the bufr
# file to be plotted must be introduced. The final image is named <file>.asii.gif and stored in the working directory,
# its size set to <width> x <height>. If 0 0 is introduced, the default dimensions (1100 x 800) are used.
# Background image is taken from the IR120-BT file from L1SD product in the same path replacing ASII by L1SD.
#
# Example:
#
# execute:      python nwcpy_asii.py /home/NWCGEO/export/ASII/S_NWC_ASII-SAT_MSG3_Spain-VISIR_20151013T151500Z.bufr 0 0
# output file:  S_NWC_ASII-SAT_MSG3_Spain-VISIR_20151013T151500Z.gif
# in:           . (working directory)
#
#**************************************************************************/
#* HISTORY
#*
#*  DATE      VERSION     AUTHOR     REASONS
#* SEP 2017    1.0       AEMET/GMV   First version
#* MAR 2018    2.0       AEMET/GMV   Operational
#*
#****************************************************************************/


import matplotlib
matplotlib.use('Agg')

import netCDF4
from pybufrkit.decoder import generate_bufr_message
from pybufrkit.decoder import Decoder
from pybufrkit.dataquery import NodePathParser, DataQuerent
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import os
import subprocess
import sys
import datetime
from matplotlib.offsetbox import (OffsetImage, AnnotationBbox)
from matplotlib.pyplot import cm


# Current date
now = datetime.datetime.now()

# Usage
if len(sys.argv) != 4:
    print "       Usage: python nwcpy_asii.py <path/file.bufr> <width> <height>"
    print "       Example: python nwcpy_asii.py /home/NWCGEO/export/ASII/S_NWC_ASII-SAT_MSG3_Spain-VISIR_20151013T151500Z.bufr 0 0"
    quit(2)

# Input and output directories
inDir = os.path.dirname(sys.argv[1])
print "inDir=", inDir
outDir = os.getcwd()
print "outDir=", outDir
# Input file
file = os.path.basename(sys.argv[1])

# ---------------- #
#  ASII bufr file  #
# ---------------- #
asiiFile = os.path.join(inDir, file)
print 'File ', asiiFile

# ------------------------------------------------------------ #
# Opens and reads L1SD data for background image (IR120 band)
# ------------------------------------------------------------ #
l1sdFile = asiiFile.replace('ASII', 'L1SD')
l1sdFile = l1sdFile.replace('_L1SD-SAT_', '_IR120-BT_')
l1sdFile = l1sdFile.replace('_L1SD-SAT+NWP_', '_IR120-BT_')
l1sdFile = l1sdFile.replace('.bufr', '.nc')
print "L1SD file", l1sdFile

l1sdNC4 = netCDF4.Dataset(l1sdFile, 'r')

prod = l1sdNC4.variables['data'][:]
print "   data type:", prod.dtype, "| max =", np.amax(prod), " min =", np.amin(prod)
print "   prod size", prod.shape
prodVar = l1sdNC4.variables['data']
validRange = prodVar.getncattr("valid_range")
print "   validRange", validRange[0], " ", validRange[1]
prodName = prodVar.getncattr("long_name")
prodUnit = prodVar.getncattr("units")
if prodUnit == "unitless":
    unitStr = ''
elif prodUnit == '1':
    unitStr = ' (%)'
else:
    unitStr = ' (' + prodUnit + ')'
fillValue = prodVar.getncattr("_FillValue")

# L1SD palette (inverse grayscale)
prodCmap = cm.Greys

# ----------------------- #
# Image frame dimensions  #
# ----------------------- #
if sys.argv[2] == '0':
    width = float(1100)
    height = float(800)
else:
    width = float(sys.argv[2])
    height = float(sys.argv[3])
print "   wxh ", width, "x", height

# -------------------------------------------------- #
# Reads gdal projection parameters from L1SD netCDF  #
# -------------------------------------------------- #
geodef = l1sdNC4.getncattr('gdal_projection')
proj = str(geodef[6:10])
a = float(geodef[14:23])
b = float(geodef[32:41])
lon_0 = float(geodef[54])
h = float(geodef[66:74])

lowleftx = l1sdNC4.getncattr('gdal_xgeo_up_left')
uprighty = l1sdNC4.getncattr('gdal_ygeo_up_left')
uprightx = l1sdNC4.getncattr('gdal_xgeo_low_right')
lowleftty = l1sdNC4.getncattr('gdal_ygeo_low_right')

# ----------------------- #
# Plots the image frame   #
# ----------------------- #
myDpi = 100  # image resolution
fig, ax = plt.subplots(figsize=(width / myDpi, height / myDpi), dpi=myDpi)

# Includes NWCSAF logo in the image
imNWCSAF = plt.imread(os.environ['NWCPY_PATH'] + '/logoNWCSAF.png')
imboxNWCSAF = OffsetImage(imNWCSAF, zoom=width/1100-width/2000)
imboxNWCSAF.image.axes = ax
abNWCSAF = AnnotationBbox(imboxNWCSAF, xy=(0.998, 0.996),
                          xycoords='axes fraction',
                          boxcoords="offset points",
                          pad=0, box_alignment=(1, 1), frameon=False)
ax.add_artist(abNWCSAF)

# Map projection definition
m = Basemap(lon_0=lon_0, llcrnrx=lowleftx, llcrnry=lowleftty, urcrnrx=uprightx, urcrnry=uprighty,
            projection=proj, resolution='l', area_thresh=250, satellite_height=h, rsphere=(a,b))
m.drawcoastlines(linewidth=1.3, color='white')
m.drawcountries(linewidth=0.8, color='white')

# PLot L1SD data
Vmin = np.amin(prod)
Vmax = np.amax(prod)
m.imshow(prod, origin='upper', vmin=Vmin, vmax=Vmax, cmap=prodCmap)

# -------------------------------- #
# Reads the data variable for ASII #
# -------------------------------- #
decoder = Decoder()
print "Opening BUFR file..."
# Opens ASII bufr file
with open(asiiFile, 'rb') as fr:
    i = 1
    lat = []
    lon = []
    rep_factor = []
    vars = []
    # Loop for all messages
    for bufr_message in generate_bufr_message(decoder, fr.read()):
        print 'Reading message number', i, '...'
        lat = np.append(lat, DataQuerent(NodePathParser()).query(bufr_message, '005002').all_values())
        lon = np.append(lon, DataQuerent(NodePathParser()).query(bufr_message, '006002').all_values())
        rep_factor = np.append(rep_factor, DataQuerent(NodePathParser()).query(bufr_message, '031002').all_values())
        vars = np.append(vars, DataQuerent(NodePathParser()).query(bufr_message, '000013').all_values())
        print ' ', vars[i-1]
        print '  ndata:', int(rep_factor[i - 1])
        i = i + 1
fr.close()

# Obtains pixel coordinates
x, y = m(lon, lat)

# ------------------ #
# Plotting ASII data #
# ------------------ #
colors = np.array(['red', 'blue', 'purple', 'magenta', 'cyan', 'cyan', 'yellow', 'yellow', 'magenta', 'blue', 'green',
                   'yellow', 'yellow', 'magenta', 'blue', 'blue', 'yellow', 'yellow', 'yellow', 'black'])
markers = np.array([r'$\mathsf{w}$', r'$\mathsf{c}$', r'$\mathsf{c}$', r'$\mathsf{c,jet}$', r'$\mathsf{ec}$',
                    r'$\mathsf{DI}$', r'$\mathsf{z}$', r'$\mathsf{m,cb,}$', r'$\mathsf{m,cb,}$', r'$\mathsf{m,cb,}$',
                    r'$\mathsf{o}$', r'$\mathsf{co}$', r'$\mathsf{\sim}$', r'$\mathsf{\sim}$', r'$\mathsf{\sim}$',
                    r'$\mathsf{lw}$', r'$\mathsf{m}$', r'$\mathsf{L}$', r'$\mathsf{F}$', r'$\mathsf{cnv}$'])
marker_size = np.array([2.0, 2.0, 2.0, 6.0, 3.0, 2.5, 2.0, 8.0, 8.0, 8.0, 2.0,
                        3.0, 2.0, 2.0, 2.0, 3.0, 2.4, 2.4, 2.4, 4.0])
label = np.array(['warm front', 'cold front', 'cold front under warm air adv', 'front intensification by jet str',
                  'enhanced cumulus', 'dry intrusion', 'cold air cloudiness', 'MCS,Cb (mature stage)',
                  'MCS,Cb (developing stage)', 'MCS,Cb (decaying stage)', 'occlusion', 'comma cloudiness',
                  'non-developing wave', 'developing wave', 'upper wave', 'upper level low', 'MCS', 'lee cloudiness',
                  'fibre', 'embedded convective cell'])
ini = 0
for var in vars:
    sel = np.where(vars == var)

    if int(rep_factor[sel]) != 0:
        fin = ini + int(rep_factor[sel])
        xS = x[ini:fin]
        yS = y[ini:fin]
        ini = fin

        # Plot each variable with the appropriate marker
        plt.plot(xS, yS, linestyle='None', marker=markers[sel][0], markersize=marker_size[sel][0]*(1.5*height/800),
                 mew=0, mfc=colors[sel][0])

# Plot legend
for i in xrange(20):  # 20 variables
    plt.plot(lowleftx, lowleftty, linestyle='None', marker=markers[i], markersize=marker_size[i]*(1.5*height/800),
             mew=0, mfc=colors[i], label=label[i])
legend = plt.legend(numpoints=1, markerscale=1.6, bbox_to_anchor=(0., -0.02, 1., -0.02), loc=2, ncol=4,
                    mode="expand", borderaxespad=0.5, fontsize=(width / 1100) * 8)
legend.get_frame().set_facecolor('silver')

# Title and text labels
title = file
title = title.replace('.bufr', '')
plt.title(title, size=(width/1100)*8)
plt.xlabel('NWC GEO ASII', size=(width/1100)*11)
bbox_props = dict(boxstyle="square", fc='white', lw=0.5)
plt.annotate(s='COPYRIGHT ' + now.strftime("%Y") + ", EUMETSAT. All rights reserved", xy=(0.003,0.994),
             size=(width/1100)*5, xycoords="axes fraction", family='monospace', bbox=bbox_props, horizontalalignment='left',
             verticalalignment='top')

# -------------------------------------- #
# Saves the image and closes the figure  #
# -------------------------------------- #
outFile = outDir + '/' + title + '.asii.gif'
print "Writing ", outFile
plt.savefig(outFile, format='png', dpi=myDpi, bbox_inches='tight')  # Python does not support GIF format
plt.close()
# Conversion to GIF calling ImageMagick
subprocess.call("convert " + outFile + " gif:" + outFile, shell=True)
quit()
