#*************************************************************************/
#**
#* \file nwcpy_hrw.py
#* \brief Python visualizer for the NWC/GEO HRW winds
#*
#* \par Copyright 2017, EUMETSAT, All Rights Reserved
#*
#* \par PRODUCED BY :  GMV Aerospace and Defence S.A.U.
#*                     This SW was developed by GMV within the
#*                     context of the EUMETSAT Satellite Application
#*                     Facility Co-operation Agreement for Support to
#*                     Nowcasting and Very Short Range Forecasting,
#*                     dated on 29 June 2011 between EUMETSAT
#*                     and AEMET.
#*
#* \par PROJECT     :  NWC SAF
#*
#* \par UNIT NAME   :  NWCPY
#*
#* \par FILE        :  nwcpy_hrw.py
#*
#* \par TYPE        :  source
#*
#* \par FUNCTION    :
# Usage: python nwcpy_hrw.py <path/file.nc> <width> <height> <type> [<pressure_interval>]
#
# This script plots wind vectors for different pressure intervals from a HRW product in nc format. The complete
# path to the nc file to be plotted must be introduced. The final image is named <file>.<type>_<pressure_interval>.gif
# and stored in the working directory, its size set to <width> x <height>. If 0 0 is introduced, the default
# dimensions (1100 x 800) are used. The parameter <type> indicates the variable to be labelled, and can take the
# values p for pressure and ws for wind speed. According to this, wind vector color corresponds to the pressure
# level where it is measured or its wind speed.
# Vectors for ALL pressure levels are plotted if no <pressure_interval> is set. The allowed values to plot a specific
# pressure interval are 0 (only background), 1 (100-400hPa), 2 (400-600hPa), 3 (600-800hPa), 4 (800-1000hPa).
# Background image is taken from the IR120-BT file from L1SD product in the same path replacing HRW by L1SD, and is
# only plotted if <pressure_interval>=0 or not set. Plots for the other intervals do not include any background.
#
# Example:
#
# execute:      python nwcpy_hrw.py /home/NWCGEO/export/HRW/S_NWC_HRW_MSG3_Spain-BS_20151013T150000Z.nc 0 0 p 1
# output file:  S_NWC_HRW_MSG3_Spain-BS_20151013T150000Z.gif (100-400 hPa winds as a function of pressure level)
# in:           . (working directory)
#
#**************************************************************************/
#* HISTORY
#*
#*  DATE      VERSION     AUTHOR     REASONS
#* SEP 2017    1.0       AEMET/GMV   First version
#* NOV 2017    1.1         GMV       Posibility of plotting different pressure intervals.
#                                    Vectors as a function of pressure interval or wind speed
#* MAR 2018    2.0       AEMET/GMV   Operational
#****************************************************************************/


import matplotlib
matplotlib.use('Agg')

import netCDF4
import h5py
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
from os import path
import os
import subprocess
import re
import sys
import datetime
from matplotlib.offsetbox import (OffsetImage, AnnotationBbox)
from matplotlib.pyplot import cm

# Current date and time
now = datetime.datetime.now()

print h5py.version.info

# Usage
if len(sys.argv) < 5:
    print "       Usage: python nwcpy_hrw.py <path/file.nc> <width> <height> <type> [<pressure_interval>]"
    print "              where  type = p (pressure level), ws (wind speed)"
    print "                     pressure_interval = 0 (only background), 1 (100-400hPa), 2 (400-600hPa), 3 (600-800hPa), 4 (800-1000hPa)"
    print "       Example: python nwcpy_hrw.py /home/NWCGEO/export/HRW/S_NWC_HRW_MSG3_Spain-BS_20151013T150000Z.nc 0 0 p 1"
    quit(2)

# Input and output directories
inDir = os.path.dirname(sys.argv[1])
print "inDir=", inDir
outDir = os.getcwd()
print "outDir=", outDir
# Input file
file = os.path.basename(sys.argv[1])

# --------------- #
# Opens HRW data  #
# --------------- #
hrwFile = os.path.join(inDir, file)
print 'File', hrwFile
# Opens HRW input file
pf5 = h5py.File(hrwFile, 'r')

# Input variable for plot type
variables = {'p': 'pressure level', 'ws': 'wind speed'}
type = sys.argv[4]
if type == 'p' or type == 'ws':
    print "   plot type:", variables[type]
else:
    print 'ERROR: Invalid plot type "', type, '"'
    quit(2)

# Input variable indicating pressure interval to be plotted
intervals = ('background', '100-400hPa', '400-600hPa', '600-800hPa', '800-1000hPa')
if len(sys.argv) == 6:
    pres_int = sys.argv[5]
    print "   pressure interval: ", intervals[int(pres_int)]
else:
    pres_int = 'all'
    print "   all pressure intervals"

# Transparent background for individual pressure intervals
if pres_int == 'all' or pres_int == '0':
    transparency_gif = ''
else:
    transparency_gif = ' -flatten -transparent white'

# Array containing the required variables
vars = ('wind_hrvis','wind_ir108','wind_ir120','wind_vis06','wind_vis08','wind_wv062','wind_wv073')

# ------------------------------------------------------------ #
# Opens and reads L1SD data for background image (IR120 band)
# ------------------------------------------------------------ #
l1sdFile = hrwFile.replace('HRW', 'L1SD')
l1sdFile = l1sdFile.replace('-BS', '-VISIR')
l1sdFile = l1sdFile.replace('_L1SD_', '_IR120-BT_')
print "L1SD file", l1sdFile

l1sdNC4 = netCDF4.Dataset(l1sdFile, 'r')

prod = l1sdNC4.variables['data'][:]
print "   data type:", prod.dtype, "| max =", np.amax(prod), " min =", np.amin(prod)
print "   prod size", prod.shape
prodVar = l1sdNC4.variables['data']
validRange = prodVar.getncattr("valid_range")
print "   validRange", validRange[0], " ", validRange[1]
prodName = prodVar.getncattr("long_name")
prodUnit = prodVar.getncattr("units")
if prodUnit == "unitless":
    unitStr = ''
elif prodUnit == '1':
    unitStr = ' (%)'
else:
    unitStr = ' (' + prodUnit + ')'
fillValue = prodVar.getncattr("_FillValue")

# L1SD palette (inverse grayscale)
prodCmap = cm.Greys

# ----------------------- #
# Image frame dimensions  #
# ----------------------- #
if sys.argv[2] == '0':
    width = float(1100)
    height = float(800)
else:
    width = float(sys.argv[2])
    height = float(sys.argv[3])
print "   wxh ", width, "x", height

# -------------------------------------------------- #
# Reads gdal projection parameters from L1SD netCDF  #
# -------------------------------------------------- #
geodef = l1sdNC4.getncattr('gdal_projection')
proj = str(geodef[6:10])
a = float(geodef[14:23])
b = float(geodef[32:41])
lon_0 = float(geodef[54])
h = float(geodef[66:74])

lowleftx = l1sdNC4.getncattr('gdal_xgeo_up_left')
uprighty = l1sdNC4.getncattr('gdal_ygeo_up_left')
uprightx = l1sdNC4.getncattr('gdal_xgeo_low_right')
lowleftty = l1sdNC4.getncattr('gdal_ygeo_low_right')

# ----------------------- #
# Plots the image frame   #
# ----------------------- #
myDpi = 100  # image resolution
fig, ax = plt.subplots(figsize=(width / myDpi, height / myDpi), dpi=myDpi)

# Includes NWCSAF logo in the image
imNWCSAF = plt.imread(os.environ['NWCPY_PATH'] + '/logoNWCSAF.png')
imboxNWCSAF = OffsetImage(imNWCSAF, zoom=width/1100-width/2000)
imboxNWCSAF.image.axes = ax
abNWCSAF = AnnotationBbox(imboxNWCSAF, xy=(0.998, 0.996),
                          xycoords='axes fraction',
                          boxcoords="offset points",
                          pad=0, box_alignment=(1, 1), frameon=False)
ax.add_artist(abNWCSAF)


# Plots L1SD data only if required (pres_int=0)
if pres_int == 'all' or pres_int == '0':
    Vmin = np.amin(prod)
    Vmax = np.amax(prod)
    # Map projection definition
    m = Basemap(lon_0=lon_0, llcrnrx=lowleftx, llcrnry=lowleftty, urcrnrx=uprightx, urcrnry=uprighty,
                projection=proj, resolution='l', area_thresh=250, satellite_height=h, rsphere=(a,b))
    m.drawcoastlines(linewidth=1.3, color='white')
    m.drawcountries(linewidth=0.8, color='white')
    m.imshow(prod, origin='upper', vmin=Vmin, vmax=Vmax, cmap=prodCmap)
else:
    m = Basemap(lon_0=lon_0, llcrnrx=lowleftx, llcrnry=lowleftty, urcrnrx=uprightx, urcrnry=uprighty,
                projection=proj, resolution='l', satellite_height=h, rsphere=(a,b))
    m.set_axes_limits()

# -------------------------------- #
# Reads the data variable for HRW  #
# -------------------------------- #
firstData = True
for wind in vars:

    windData = pf5[wind]

    if windData.size > 0:
        latO = windData['lat']
        lonO = windData['lon']
        pressureO = windData['pressure']
        wind_speedO = windData['wind_speed']
        # Convert to knots
        wind_speedO*= 1.94384
        wind_directionO = windData['wind_direction']
        cloud_typeO = windData['cloud_type']
        confidence_foreO = windData['confidence_fore']

        if firstData:
            lat = latO
            lon = lonO
            pressure = pressureO
            wind_speed = wind_speedO
            wind_direction = wind_directionO
            cloud_type = cloud_typeO
            confidence_fore = confidence_foreO
            firstData = False
        else:
            lat = np.append(lat, latO)
            lon = np.append(lon, lonO)
            pressure = np.append(pressure, pressureO)
            wind_speed = np.append(wind_speed, wind_speedO)
            wind_direction = np.append(wind_direction, wind_directionO)
            cloud_type = np.append(cloud_type, cloud_typeO)
            confidence_fore = np.append(confidence_fore, confidence_foreO)

x, y = m(lon, lat)

# Calculates wind componets referred to the North
u = wind_speed*np.cos((270.0-wind_direction)/180.0*np.pi)
v = wind_speed*np.sin((270.0-wind_direction)/180.0*np.pi)

if type == 'p':
    # Defines pressure-level intervals
    conForMin = (70, 70, 70, 70, 70, 70, 75, 75, 75)
    colors = ('magenta', 'red', 'orange', 'yellow', 'green', '#94b001', 'cyan', 'blue', 'purple')
    for ipres in xrange(9):
        presMin = ipres*10000+10000      # 100hPa
        presMax = (ipres+1)*10000+10000  # 1000hPa
        # Cloudy
        label = str(presMin / 100) + '-' + str(presMax / 100) + ' hPa'
        sel = np.where((pressure>presMin) * (pressure<=presMax) * (confidence_fore>=conForMin[ipres]) *
                       (confidence_fore<=100) * (cloud_type>=5) * (cloud_type<=15))
        xS = x[sel]
        yS = y[sel]
        uS = u[sel]
        vS = v[sel]

        # Decluttering algorithm--------------------
        # We create an x,y grid. Number of points n the grid
        nG = 40
        if len(xS) != 0:   # at least one point with data is required
            xG, yG = np.meshgrid(np.linspace(np.amin(xS), np.amax(xS), num=nG),
                                 np.linspace(np.amin(yS), np.amax(yS), num=nG))
            # distance from wind points to grid points
            dist = (np.expand_dims(xG.flatten(), axis=0) - np.expand_dims(xS, axis=1)) ** 2\
                   + (np.expand_dims(yG.flatten(), axis=0) - np.expand_dims(yS, axis=1)) ** 2
            # only the nearest wind point is plotted
            distMin = np.argmin(dist, axis=0)
            xS = xS[distMin]
            yS = yS[distMin]
            uS = uS[distMin]
            vS = vS[distMin]
        # End decluttering algorithm----------------
        else:
            xS = yS = uS = vS = 0

        # ------------------ #
        # Plotting HRW data  #
        # ------------------ #
        if pres_int == 'all':
            plt.barbs(xS, yS, uS, vS, color=colors[ipres], length=7, linewidth=0.1*(width/1100), label=label)
            # Plotting legend
            plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=5, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                       frameon=False)
        if pres_int == '0':
            plt.barbs(np.NaN, np.NaN, np.NaN, np.NaN, color='none', label=label)
            # Plotting legend frame only
            legend = plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=5, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                                frameon=False)
            plt.setp(legend.get_texts(), color='none')
        if pres_int == '1':
            if ipres <= 2:
                plt.barbs(xS, yS, uS, vS, color=colors[ipres], length=7, linewidth=0.1*(width/1100), label=label, antialiased=False)
                # Plotting legend
                plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=2, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                           frameon=False)
        if pres_int == '2':
            if ipres == 3 or ipres == 4:
                plt.barbs(xS, yS, uS, vS, color=colors[ipres], length=7, linewidth=0.1*(width/1100), label=label, antialiased=False)
                # Plotting legend
                plt.legend(bbox_to_anchor=(0.32, -0.04), loc=2, ncol=1, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                           frameon=False)
        if pres_int == '3':
            if ipres == 5 or ipres == 6:
                plt.barbs(xS, yS, uS, vS, color=colors[ipres], length=7, linewidth=0.1*(width/1100), label=label, antialiased=False)
                # Plotting legend
                plt.legend(bbox_to_anchor=(0.48, -0.04), loc=2, ncol=1, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                           frameon=False)
        if pres_int == '4':
            if ipres == 7 or ipres == 8:
                plt.barbs(xS, yS, uS, vS, color=colors[ipres], length=7, linewidth=0.1*(width/1100), label=label, antialiased=False)
                # Plotting legend
                plt.legend(bbox_to_anchor=(0.64, -0.04), loc=2, ncol=1, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                           frameon=False)
if type == 'ws':
    # Defines wind speed intervals
    conForMin = (75, 75, 75, 75, 75, 75, 70, 70, 70, 70)
    colors = ('purple', 'magenta', 'blue', 'cyan', '#94b001', 'green', 'yellow', 'pink', 'orange', 'red')
    speedMin = (0, 20, 40, 60, 80, 100, 120, 160, 200, 240)    # km/h
    speedMax = (19, 39, 59, 79, 99, 119, 159, 199, 239, 9999)  # km/h
    for ispeed in xrange(10):
        # Cloudy
        if ispeed == 9:
            label = '>= ' + str(speedMin[ispeed]) + ' km/h'
        else:
            label = str(speedMin[ispeed]) + '-' + str(speedMax[ispeed]) + ' km/h'
        # pressure intervals
        if pres_int == 'all':
            presMin = 10000
            presMax = 100000
        if pres_int == '0':
            presMin = 0
            presMax = 0
        if pres_int == '1':
            presMin = 10000  # *100hPa
            presMax = 40000
        if pres_int == '2':
            presMin = 40000
            presMax = 60000
        if pres_int == '3':
            presMin = 60000
            presMax = 80000
        if pres_int == '4':
            presMin = 80000
            presMax = 100000
        # conversion to km/h (*1.852)
        sel = np.where((pressure > presMin) * (pressure <= presMax) * ((wind_speed*1.852)>speedMin[ispeed]) *
                        ((wind_speed*1.852)<=speedMax[ispeed]) * (confidence_fore>=conForMin[ispeed]) *
                        (confidence_fore<=100) * (cloud_type>=5) * (cloud_type<=15))
        xS = x[sel]
        yS = y[sel]
        uS = u[sel]
        vS = v[sel]

        # Decluttering algorithm--------------------
        # We create an x,y grid. Number of points n the grid
        nG = 40
        if len(xS) != 0:   # at least one point with data is required
            xG, yG = np.meshgrid(np.linspace(np.amin(xS), np.amax(xS), num=nG),
                                 np.linspace(np.amin(yS), np.amax(yS), num=nG))
            dist = (np.expand_dims(xG.flatten(), axis=0) - np.expand_dims(xS, axis=1)) ** 2 +\
                   (np.expand_dims(yG.flatten(), axis=0) - np.expand_dims(yS, axis=1)) ** 2
            distMin = np.argmin(dist, axis=0)
            xS = xS[distMin]
            yS = yS[distMin]
            uS = uS[distMin]
            vS = vS[distMin]
        # End decluttering algorithm----------------
        else:
            xS = yS = uS = vS = 0

        # ------------------ #
        # Plotting HRW data  #
        # ------------------ #
        if pres_int == '0':
            # Plotting legend frame only
            plt.barbs(np.NaN, np.NaN, np.NaN, np.NaN, color='none', label=label)
            legend = plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=5, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                                frameon=False)
            plt.setp(legend.get_texts(), color='none')
        else:
            plt.barbs(xS, yS, uS, vS, color=colors[ispeed], length=7, linewidth=0.1*(width/1100), label=label, antialiased=False)
            # Plotting legend
            plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=5, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                       frameon=False)

# Title and text labels
title = file
title = title.replace('.nc', '')
plt.title(title, size=(width/1100)*8)
plt.xlabel('NWC GEO HRW', size=(width/1100)*11)
bbox_props = dict(boxstyle="square", fc='white', lw=0.5)
plt.annotate(s='COPYRIGHT ' + now.strftime("%Y") + ", EUMETSAT. All rights reserved", xy=(0.003,0.994),
             size=(width/1100)*5, xycoords="axes fraction", family='monospace', bbox=bbox_props, horizontalalignment='left',
             verticalalignment='top')

# -------------------------------------- #
# Saves the image and closes the figure  #
# -------------------------------------- #
if pres_int == 'all':
    outFile = outDir + '/' + title + '.' + type + '_all.gif'
else:
    outFile = outDir + '/' + title + '.' + type + '_' + intervals[int(pres_int)] + '.gif'
print "Writing ", outFile
plt.savefig(outFile, format='png', dpi=myDpi, bbox_inches='tight')  # Python does not support GIF format
plt.close()
# Conversion to GIF calling ImageMagick
subprocess.call("convert " + outFile + transparency_gif + " gif:" + outFile, shell=True)
quit()
