#*************************************************************************/
#**
#* \file nwcpy_hrw_traj.py
#* \brief Python visualizer for the NWC/GEO HRW trajectories
#*
#* \par Copyright 2017, EUMETSAT, All Rights Reserved
#*
#* \par PRODUCED BY :  GMV Aerospace and Defence S.A.U.
#*                     This SW was developed by GMV within the
#*                     context of the EUMETSAT Satellite Application
#*                     Facility Co-operation Agreement for Support to
#*                     Nowcasting and Very Short Range Forecasting,
#*                     dated on 29 June 2011 between EUMETSAT
#*                     and AEMET.
#*
#* \par PROJECT     :  NWC SAF
#*
#* \par UNIT NAME   :  NWCPY
#*
#* \par FILE        :  nwcpy_hrw_traj.py
#*
#* \par TYPE        :  source
#*
#* \par FUNCTION    :
# Usage: python nwcpy_hrw_traj.py <path/file.nc> <width> <height> <n_slots> [<pressure_interval>]
#
# This script plots wind trajectories for different pressure intervals from a HRW product in nc format. The complete
# path to the nc file to be plotted must be introduced. The final image is named
# <file>.traj_<pressure_interval>_<n_slots>slots.gif and stored in the working directory, its size set to
# <width> x <height>. : If 0 0 is introduced, the default dimensions (1100 x 800) are used.
# Trajectory color corresponds to the pressure level where it is measured. The number of slots to be used to plot
# trajectories is <n_slots>. If the number of predecessor winds in the HRW nc file + 1 is lower than <n_slots>,
# an error is given. Trajectories for ALL pressure levels are plotted if no <pressure_interval> is set. The allowed
# values to plot a specific pressure interval are 0 (only background), 1 (100-400hPa), 2 (400-600hPa), 3 (600-800hPa),
# 4 (800-1000hPa). Background image is taken from the IR120-BT file from L1SD product in the same path replacing
# HRW by L1SD, and is only plotted if <pressure_interval>=0 or not set. Plots for the other intervals do not include
# any background.
#
# Example:
#
# execute:      python nwcpy_hrw_traj.py /home/NWCGEO/export/HRW/S_NWC_HRW_MSG3_Spain-BS_20151013T154500Z.nc 0 0 4 1
# output file:  S_NWC_HRW_MSG3_Spain-BS_20151013T154500Z.gif (100-400 hPa trajectories)
# in:           . (working directory)
#
#**************************************************************************/
#* HISTORY
#*
#*  DATE      VERSION     AUTHOR     REASONS
#* NOV 2017    1.0       AEMET/GMV   First version
#* MAR 2018    2.0       AEMET/GMV   Operational
#*
#****************************************************************************/


import matplotlib
matplotlib.use('Agg')

import netCDF4
import h5py
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
from os import path
import subprocess
import os
import re
import sys
import datetime
from matplotlib.offsetbox import (OffsetImage, AnnotationBbox)
from matplotlib.pyplot import cm

# Current date and time
now = datetime.datetime.now()

print h5py.version.info

# Usage
if len(sys.argv) < 5:
    print "       Usage: python nwcpy_hrw_traj.py <path/file.nc> <width> <height> <n_slots> [<pressure_interval>]"
    print "              where  n_slots >= 2"
    print "                     pressure_interval = 0 (only background), 1 (100-400hPa), 2 (400-600hPa), 3 (600-800hPa), 4 (800-1000hPa)"
    print "       Example: python nwcpy_hrw_traj.py /home/NWCGEO/export/HRW/S_NWC_HRW_MSG3_Spain-BS_20151013T154500Z.nc 0 0 4 1"
    quit(2)

# Input and output directories
inDir = os.path.dirname(sys.argv[1])
print "inDir=", inDir
outDir = os.getcwd()
print "outDir=", outDir
# Input file
file = os.path.basename(sys.argv[1])

# -------------- #
#  HRW nc file   #
# -------------- #
hrwFile = os.path.join(inDir, file)
print 'File', hrwFile

# Input variable for slot number
nslots = int(sys.argv[4])
if nslots >= 2:
    print "   number of slots:", nslots
else:
    print 'ERROR: Invalid slot number "', nslots, '" . Must be >=2'
    quit(2)

# Input variable indicating pressure interval to be plotted
intervals = ('background', '100-400hPa', '400-600hPa', '600-800hPa', '800-1000hPa')
if len(sys.argv) == 6:
    pres_int = sys.argv[5]
    print "   pressure interval: ", intervals[int(pres_int)]
else:
    pres_int = 'all'
    print "   all pressure intervals"

# Transparent background for individual pressure intervals
if pres_int == 'all' or pres_int == '0':
    transparency_gif = ''
else:
    transparency_gif = ' -flatten -transparent white'

# Array containing the required variables
vars = ('wind_hrvis','wind_ir108','wind_ir120','wind_vis06','wind_vis08','wind_wv062','wind_wv073')

# ------------------------------------------------------------ #
# Opens and reads L1SD data for background image (IR120 band)
# ------------------------------------------------------------ #
l1sdFile = hrwFile.replace('HRW', 'L1SD')
l1sdFile = l1sdFile.replace('-BS', '-VISIR')
l1sdFile = l1sdFile.replace('_L1SD_', '_IR120-BT_')
print "L1SD file", l1sdFile

l1sdNC4 = netCDF4.Dataset(l1sdFile, 'r')

prod = l1sdNC4.variables['data'][:]
print "   data type:", prod.dtype, "| max =", np.amax(prod), " min =", np.amin(prod)
print "   prod size", prod.shape
prodVar = l1sdNC4.variables['data']
validRange = prodVar.getncattr("valid_range")
print "   validRange", validRange[0], " ", validRange[1]
prodName = prodVar.getncattr("long_name")
prodUnit = prodVar.getncattr("units")
if prodUnit == "unitless":
    unitStr = ''
elif prodUnit == '1':
    unitStr = ' (%)'
else:
    unitStr = ' (' + prodUnit + ')'
fillValue = prodVar.getncattr("_FillValue")

# L1SD palette (inverse grayscale)
prodCmap = cm.Greys

# ----------------------- #
# Image frame dimensions  #
# ----------------------- #
if sys.argv[2] == '0':
    width = float(1100)
    height = float(800)
else:
    width = float(sys.argv[2])
    height = float(sys.argv[3])
print "   wxh ", width, "x", height
print " "

# -------------------------------------------------- #
# Reads gdal projection parameters from L1SD netCDF  #
# -------------------------------------------------- #
geodef = l1sdNC4.getncattr('gdal_projection')
proj = str(geodef[6:10])
a = float(geodef[14:23])
b = float(geodef[32:41])
lon_0 = float(geodef[54])
h = float(geodef[66:74])

lowleftx = l1sdNC4.getncattr('gdal_xgeo_up_left')
uprighty = l1sdNC4.getncattr('gdal_ygeo_up_left')
uprightx = l1sdNC4.getncattr('gdal_xgeo_low_right')
lowleftty = l1sdNC4.getncattr('gdal_ygeo_low_right')

# ----------------------- #
# Plots the image frame   #
# ----------------------- #
myDpi = 100  # image resolution
fig, ax = plt.subplots(figsize=(width / myDpi, height / myDpi), dpi=myDpi)

# Includes NWCSAF logo in the image
imNWCSAF = plt.imread(os.environ['NWCPY_PATH'] + '/logoNWCSAF.png')
imboxNWCSAF = OffsetImage(imNWCSAF, zoom=width/1100-width/2000)
imboxNWCSAF.image.axes = ax
abNWCSAF = AnnotationBbox(imboxNWCSAF, xy=(0.998, 0.996),
                          xycoords='axes fraction',
                          boxcoords="offset points",
                          pad=0.5, box_alignment=(1, 1), frameon=False)
ax.add_artist(abNWCSAF)


# Plots L1SD data only if required (pres_int=0)
if pres_int == 'all' or pres_int == '0':
    Vmin = np.amin(prod)
    Vmax = np.amax(prod)
    # Map projection definition
    m = Basemap(lon_0=lon_0, llcrnrx=lowleftx, llcrnry=lowleftty, urcrnrx=uprightx, urcrnry=uprighty,
                projection=proj, resolution='l', area_thresh=250, satellite_height=h, rsphere=(a,b))
    m.drawcoastlines(linewidth=1.3, color='white')
    m.drawcountries(linewidth=0.8, color='white')
    m.imshow(prod, origin='upper', vmin=Vmin, vmax=Vmax, cmap=prodCmap)
else:
    m = Basemap(lon_0=lon_0, llcrnrx=lowleftx, llcrnry=lowleftty, urcrnrx=uprightx, urcrnry=uprighty,
                projection=proj, resolution='l', satellite_height=h, rsphere=(a,b))
    m.set_axes_limits()

# -------------------------------- #
# Reads the data variable for HRW  #
# -------------------------------- #
firstData = True

for wind in vars:
    # ------------------------------------- TRAJECTORIES -----------------------------------------#
    # Execute system command 'ncdump'
    output = subprocess.check_output("ncdump -v " + wind + " " + hrwFile, shell=True)
    # Initialise the required parameters
    cloud_typeO = []
    confidence_foreO = []
    latO = []
    lonO = []
    pressureO = []
    traj_latO = []
    traj_lonO = []
    traj_pressureO = []
    # Look for a specific variable
    name = wind + ' ='
    pos = output.find(name)
    if pos > 0:
        # Select the data
        print 'Extracting trajectory data for band', wind, '...'
        data = output[pos + 14:len(output) - 5]  # data for the variable, skipping variable name and blank spaces
        # Write the data in a txt file
        fw = open('trajectories.txt', 'w')
        fw.write(data)
        fw.close()
        # Read the required parameters from the txt
        fr = open('trajectories.txt', 'r')
        for line in fr:
            line = line.split(', ')
            n_pred_winds = line[24]  # element 25 of Wind structure
            if (int(n_pred_winds) + 1) >= nslots:
                # cloud_type: element 27 of Wind structure
                cloud_typeO = np.append(cloud_typeO, int(line[26]))
                # confidence_fore: element 17 of Wind structure
                confidence_foreO = np.append(confidence_foreO, int(line[16]))
                # wind latitude: element 8 of Wind structure
                latO = np.append(latO, float(line[7]))
                # trajectory latitude: element 2 of Segment structure no.'nslots'
                traj_latO = np.append(traj_latO, float(line[29 + (nslots - 1) * 16]))
                # wind longitude: element 9 of Wind structure
                lonO = np.append(lonO, float(line[8]))
                # trajectory longitude: element 3 of Segment structure no.'nslots'
                traj_lonO = np.append(traj_lonO, float(line[30 + (nslots - 1) * 16]))
                # wind pressure: element 12 of Wind structure
                pressureO = np.append(pressureO, float(line[11]))
                # trajectory pressure: element 6 of Segment structure no.'nslots'
                traj_pressureO = np.append(traj_pressureO, float(line[33 + (nslots - 1) * 16]))

        fr.close()
    else:
        print 'Warning: No data available for band', wind

    if firstData:
        lat = latO
        lon = lonO
        pressure = pressureO
        cloud_type = cloud_typeO
        confidence_fore = confidence_foreO
        traj_lat = traj_latO
        traj_lon = traj_lonO
        traj_pressure = traj_pressureO
        firstData = False
    else:
        lat = np.append(lat, latO)
        lon = np.append(lon, lonO)
        pressure = np.append(pressure, pressureO)
        cloud_type = np.append(cloud_type, cloud_typeO)
        confidence_fore = np.append(confidence_fore, confidence_foreO)
        traj_lat = np.append(traj_lat, traj_latO)
        traj_lon = np.append(traj_lon, traj_lonO)
        traj_pressure = np.append(traj_pressure, traj_pressureO)

# Removes txt file
subprocess.call(["rm", "trajectories.txt"])

# Evaluates if trajectories exist
print ' '
if len(traj_lat) == 0:
    print 'ERROR: Not predecessors enough to calculate', float(nslots)/4, 'h trajectories'
    quit(2)
# ---------------------------------- END TRAJECTORIES ------------------------------------------- #

# Obtains pixel coordinates
x, y = m(lon, lat)              # winds
xt, yt = m(traj_lon, traj_lat)  # trajectories

# Defines pressure-level intervals
conForMin = (70, 70, 70, 70, 70, 70, 75, 75, 75)
colors = ('magenta', 'red', 'orange', 'yellow', 'green', '#94b001', 'cyan', 'blue', 'purple')
print 'Plotting trajectories ...'
for ipres in xrange(9):
    presMin = ipres*10000+10000      # 100hPa
    presMax = (ipres+1)*10000+10000  # 1000hPa
    # Cloudy
    label = str(presMin / 100) + '-' + str(presMax / 100) + ' hPa'
    sel = np.where((pressure>presMin) * (pressure<=presMax) * (confidence_fore>=conForMin[ipres]) *
                   (confidence_fore<=100) * (cloud_type>=5) * (cloud_type<=15))
    xS = x[sel]
    yS = y[sel]
    xtS = xt[sel]
    ytS = yt[sel]

    # Decluttering algorithm--------------------
    # We create an x,y grid. Number of points n the grid
    nG = 50
    if len(xS) != 0:   # at least one point with data is required
        xG, yG = np.meshgrid(np.linspace(np.amin(xS), np.amax(xS), num=nG),
                             np.linspace(np.amin(yS), np.amax(yS), num=nG))
        # distance from wind points to grid points
        dist = (np.expand_dims(xG.flatten(), axis=0) - np.expand_dims(xS, axis=1)) ** 2\
               + (np.expand_dims(yG.flatten(), axis=0) - np.expand_dims(yS, axis=1)) ** 2
        # only the nearest wind point is plotted
        distMin = np.argmin(dist, axis=0)
        xS = xS[distMin]
        yS = yS[distMin]
        xtS = xtS[distMin]
        ytS = ytS[distMin]
        # End decluttering algorithm----------------

        # ------------------ #
        # Plotting HRW data  #
        # ------------------ #
        # Segment coordinates
        segx = (xS, xtS)
        segy = (yS, ytS)
    else:
        segx = segy = np.empty([1,1])

    # Pressure intervals
    if pres_int == 'all':
        plt.plot(segx, segy, color=colors[ipres], linewidth=1.2)
        plt.plot(segx[0][0], segy[0][0], color=colors[ipres], linewidth=2.0, label=label)  # plot from which legend is obtained
        # Plotting legend
        plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=5, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                   frameon=False)
    if pres_int == '0':
        plt.plot(np.NaN, np.NaN, color='none', label=label)
        # Plotting legend frame only
        legend = plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=5, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                            frameon=False)
        plt.setp(legend.get_texts(), color='none')
    if pres_int == '1':
        if ipres <= 2:
            plt.plot(segx, segy, color=colors[ipres], linewidth=0.9, antialiased=False)
            plt.plot(segx[0][0], segy[0][0], color=colors[ipres], linewidth=2.0, label=label)  # plot from which legend is obtained
            # Plotting legend
            plt.legend(bbox_to_anchor=(0., -0.04), loc=2, ncol=2, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                       frameon=False)
    if pres_int == '2':
        if ipres == 3 or ipres == 4:
            plt.plot(segx, segy, color=colors[ipres], linewidth=0.9, antialiased=False)
            plt.plot(segx[0][0], segy[0][0], color=colors[ipres], linewidth=2.0, label=label)  # plot from which legend is obtained
            # Plotting legend
            plt.legend(bbox_to_anchor=(0.32, -0.04), loc=2, ncol=1, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                       frameon=False)
    if pres_int == '3':
        if ipres == 5 or ipres == 6:
            plt.plot(segx, segy, color=colors[ipres], linewidth=0.9, antialiased=False)
            plt.plot(segx[0][0], segy[0][0], color=colors[ipres], linewidth=2.0, label=label)  # plot from which legend is obtained
            # Plotting legend
            plt.legend(bbox_to_anchor=(0.48, -0.04), loc=2, ncol=1, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                       frameon=False)
    if pres_int == '4':
        if ipres == 7 or ipres == 8:
            plt.plot(segx, segy, color=colors[ipres], linewidth=0.9, antialiased=False)
            plt.plot(segx[0][0], segy[0][0], color=colors[ipres], linewidth=2.0, label=label)  # plot from which legend is obtained
            # Plotting legend
            plt.legend(bbox_to_anchor=(0.64, -0.04), loc=2, ncol=1, borderaxespad=0.5, fontsize=(width / 1100) * 8,
                       frameon=False)

# Title and text labels
title = file
title = title.replace('.nc', '')
plt.title(title, size=(width/1100)*8)
plt.xlabel('NWC GEO HRW', size=(width/1100)*11)
bbox_props = dict(boxstyle="square", fc='white', lw=0.5)
plt.annotate(s='COPYRIGHT ' + now.strftime("%Y") + ", EUMETSAT. All rights reserved", xy=(0.003,0.994),
             size=(width/1100)*5, xycoords="axes fraction", family='monospace', bbox=bbox_props, horizontalalignment='left',
             verticalalignment='top')

# -------------------------------------- #
# Saves the image and closes the figure  #
# -------------------------------------- #
if pres_int == 'all':
    outFile = outDir + '/' + title + '.' + 'traj_all' + '_' + str(nslots) + 'slots.gif'
else:
    outFile = outDir + '/' + title + '.' + 'traj_' + intervals[int(pres_int)] + '_' + str(nslots) + 'slots.gif'
print "Writing ", outFile
plt.savefig(outFile, format='png', dpi=myDpi, bbox_inches='tight')  # Python does not support GIF format
plt.close()
# Conversion to gif using ImageMagick
subprocess.call("convert " + outFile + transparency_gif + " gif:" + outFile, shell=True)
quit()
