#*************************************************************************/
#**
#* \file nwcpy_rdt.py
#* \brief Python visualizer for the NWC/GEO RDT product
#*
#* \par Copyright 2017, EUMETSAT, All Rights Reserved
#*
#* \par PRODUCED BY :  GMV Aerospace and Defence S.A.U.
#*                     This SW was developed by GMV within the
#*                     context of the EUMETSAT Satellite Application
#*                     Facility Co-operation Agreement for Support to
#*                     Nowcasting and Very Short Range Forecasting,
#*                     dated on 29 June 2011 between EUMETSAT
#*                     and AEMET.
#*
#* \par PROJECT     :  NWC SAF
#*
#* \par UNIT NAME   :  NWCPY
#*
#* \par FILE        :  nwcpy_rdt.py
#*
#* \par TYPE        :  source
#*
#* \par FUNCTION    :
# Usage: python plot_rdt.py <path/file.nc> <width> <height>
#
# This script plots thunderstorm contours from a RDT product in nc format. The complete path to the nc file to be
# plotted must be introduced. The final image is named <file>.rdt.gif and stored in the working directory, its size set
# to <width> x <height>. If 0 0 is introduced, the default dimensions (1100 x 800) are used.
# Background image is taken from the IR120-BT file from L1SD product in the same path replacing RDT by L1SD.
#
# Example:
#
# execute:      python nwcpy_rdt.py /home/NWCGEO/export/RDT/S_NWC_RDT-CW_MSG3_Spain-VISIR_20151013T150000Z.nc 0 0
# output file:  S_NWC_RDT-CW_MSG3_Spain-VISIR_20151013T150000Z.gif
# in:           . (working directory)
#
#**************************************************************************/
#* HISTORY
#*
#*  DATE      VERSION     AUTHOR     REASONS
#* SEP 2017    1.0       AEMET/GMV   First version
#* MAR 2018    2.0       AEMET/GMV   Operational
#*
#****************************************************************************/


import matplotlib
matplotlib.use('Agg')

import netCDF4
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
from os import path
import os
import subprocess
import re
import sys
import datetime
from matplotlib.offsetbox import (OffsetImage, AnnotationBbox)
from matplotlib.pyplot import cm

# Current date
now = datetime.datetime.now()

# Usage
if len(sys.argv) != 4:
    print "       Usage: python nwcpy_rdt.py <path/file.nc> <width> <height>"
    print "       Example: python nwcpy_rdt.py /home/NWCGEO/export/RDT/S_NWC_RDT-CW_MSG3_Spain-VISIR_20151013T150000Z.nc 0 0"
    quit(2)

# Input and output directories
inDir = os.path.dirname(sys.argv[1])
print "inDir=", inDir
outDir = os.getcwd()
print "outDir=", outDir
# Input file
file = os.path.basename(sys.argv[1])

# -------------------------- #
# Opens and reads RDT data   #
# -------------------------- #
rdtFile = os.path.join(inDir, file)
print 'File', rdtFile
# Opens the netCDF product
prodNC4 = netCDF4.Dataset(rdtFile, 'r')
# Reads input variables
variables = prodNC4.variables
LatG = prodNC4.variables['LatG'][:]
LonG = prodNC4.variables['LonG'][:]
LatContour = prodNC4.variables['LatContour'][:]
LonContour = prodNC4.variables['LonContour'][:]
PhaseLife = prodNC4.variables['PhaseLife'][:]
severity = prodNC4.variables['SeverityIntensity'][:]

# Reading forecast variables
times = ('000', '015', '030', '045', '060')

LatGFct = np.zeros((LatG.shape[0], len(times)))
LonGFct = np.zeros((LonG.shape[0], len(times)))
LatGFct[:, 0] = LatG[:, 0]
LonGFct[:, 0] = LonG[:, 0]
for it in xrange(1, len(times)):
    rdtFileFct = rdtFile.replace('.nc', '_' + times[it] + '.nc')
    print "Reading ", rdtFileFct
    prodFct = netCDF4.Dataset(rdtFileFct, 'r')
    LatGFct[:, it] = prodFct.variables['LatG'][:, 0]
    LonGFct[:, it] = prodFct.variables['LonG'][:, 0]

# ------------------------------------------------------------ #
# Opens and reads L1SD data for background image (IR120 band)  #
# ------------------------------------------------------------ #
l1sdFile = rdtFile.replace('RDT-CW', 'IR120-BT')
l1sdFile = l1sdFile.replace('/RDT/', '/L1SD/')
print "L1SD file", l1sdFile, "----------"

l1sdNC4 = netCDF4.Dataset(l1sdFile, 'r')

prod = l1sdNC4.variables['data'][:]
print "   data type:", prod.dtype, "| max =", np.amax(prod), " min =", np.amin(prod)
print "   prod size", prod.shape
prodVar = l1sdNC4.variables['data']
validRange = prodVar.getncattr("valid_range")
print "   validRange", validRange[0], " ", validRange[1]
prodName = prodVar.getncattr("long_name")
prodUnit = prodVar.getncattr("units")
if prodUnit == "unitless":
    unitStr = ''
elif prodUnit == '1':
    unitStr = ' (%)'
else:
    unitStr = ' (' + prodUnit + ')'
fillValue = prodVar.getncattr("_FillValue")
# L1SD palette (inverse grayscale)
l1sdCmap = cm.Greys

# ------------------------ #
# RDT palette and legend   #
# ------------------------ #
colorPhaseLife = {0: 'yellow', 1: 'orange', 2: 'red', 3: 'purple', 4: 'blue'}
labelPhaseLife = {0: 'Triggering', 1: 'Triggering from Split', 2: 'Growing', 3: 'Maturity', 4: 'Decaying'}
flagClass = True
flagUndef = False
flag_meanings = ['Triggering',
                 'Triggering from Split',
                 'Growing',
                 'Maturity',
                 'Decaying'
                 ]
severity_meanings = ['Not defined',
                     'Low',
                     'Moderate',
                     'High',
                     'Very High'
                     ]
bounds = np.array([0, 1, 2, 3, 4, 5])
flag_values = np.array([0, 1, 2, 3, 4])

# ------------------------ #
# Image frame dimensions   #
# ------------------------ #
if sys.argv[2] == '0':
    width = float(1100)
    height = float(800)
else:
    width = float(sys.argv[2])
    height = float(sys.argv[3])
print "   wxh ", width, "x", height

# ------------------------------------------------- #
# Reads gdal projection parameters from RDT netCDF  #
# ------------------------------------------------- #
geodef = prodNC4.getncattr('gdal_projection')
proj = str(geodef[6:10])
a = float(geodef[14:23])
b = float(geodef[32:41])
lon_0 = float(geodef[54])
h = float(geodef[66:74])

lowleftx = prodNC4.getncattr('gdal_xgeo_up_left')
uprighty = prodNC4.getncattr('gdal_ygeo_up_left')
uprightx = prodNC4.getncattr('gdal_xgeo_low_right')
lowleftty = prodNC4.getncattr('gdal_ygeo_low_right')

# ----------------------- #
# Plots the image frame   #
# ----------------------- #
myDpi = 100  # image resolution
fig, ax = plt.subplots(figsize=(width / myDpi, height / myDpi), dpi=myDpi)

# Includes NWCSAF logo in the image
imNWCSAF = plt.imread(os.environ['NWCPY_PATH'] + '/logoNWCSAF.png')
imboxNWCSAF = OffsetImage(imNWCSAF, zoom=width/1100-width/2000)
imboxNWCSAF.image.axes = ax
abNWCSAF = AnnotationBbox(imboxNWCSAF, xy=(0.998, 0.996),
                          xycoords='axes fraction',
                          boxcoords="offset points",
                          pad=0, box_alignment=(1, 1), frameon=False)
ax.add_artist(abNWCSAF)

# Map projection definition
m = Basemap(lon_0=lon_0, llcrnrx=lowleftx, llcrnry=lowleftty, urcrnrx=uprightx, urcrnry=uprighty,
            projection=proj, resolution='l', area_thresh=250, satellite_height=h, rsphere=(a,b))
m.drawcoastlines(linewidth=1.3, color='white')
m.drawcountries(linewidth=0.8, color='white')

# Plots L1SD data
Vmin = np.amin(prod)
Vmax = np.amax(prod)
m.imshow(prod, origin='upper', vmin=Vmin, vmax=Vmax, cmap=l1sdCmap)

# --------------------------------------------------- #
# Loop through number of contours. Plotting RDT data  #
# --------------------------------------------------- #
for icont in xrange(LatContour.shape[0]):
    LatContInt = LatContour[icont, 0, :]
    LonContInt = LonContour[icont, 0, :]
    (xContInt, yContInt) = m(LonContInt, LatContInt)
    m.plot(xContInt, yContInt, color=colorPhaseLife[PhaseLife[icont]], linewidth=severity[icont] + 1.0)
    xFct, yFct = m(LonGFct[icont, :], LatGFct[icont, :])
    m.plot(xFct, yFct, '-', color='black')

# Title and text labels
title = file
title = title.replace('.nc', '')
plt.title(title, size=(width/1100)*8)
plt.xlabel('NWC GEO RDT-CW', size=(width/1100)*11)
bbox_props = dict(boxstyle="square", fc='white', lw=0.5)
plt.annotate(s='COPYRIGHT ' + now.strftime("%Y") + ", EUMETSAT. All rights reserved", xy=(0.003,0.994),
             size=(width/1100)*5, xycoords="axes fraction", family='monospace', bbox=bbox_props, horizontalalignment='left',
             verticalalignment='top')
# Plotting legend
for i in xrange(5):
    m.plot(lowleftx, lowleftty, color=colorPhaseLife[i], linewidth=3, label=labelPhaseLife[i])
legend = plt.legend(bbox_to_anchor=(0., -0.02, 1., -0.02), loc=2, ncol=5, mode="expand", borderaxespad=0.5, fontsize=(width / 1100) * 8,)
legend.get_frame().set_facecolor('silver')

# -------------------------------------- #
# Saves the image and closes the figure  #
# -------------------------------------- #
outFile = outDir + '/' + title + '.rdt.gif'
print "Writing ", outFile
plt.savefig(outFile, format='png', dpi=myDpi, bbox_inches='tight')  # Python does not support GIF format
plt.close()
# Conversion to GIF calling ImageMagick
subprocess.call("convert " + outFile + " gif:" + outFile, shell=True)
quit()
